<?php
namespace nur\sery\php\time;

use DateTimeInterface;
use DateTimeZone;
use InvalidArgumentException;

/**
 * Class DateTime: une date et une heure
 *
 * @property-read int $year
 * @property-read int $month
 * @property-read int $day
 * @property-read int $hour
 * @property-read int $minute
 * @property-read int $second
 * @property-read int $wday
 * @property-read int $wnum
 * @property-read string $timezone
 * @property-read string $datetime
 * @property-read string $date
 */
class DateTime extends \DateTime {
  const DEFAULT_FORMAT = "d/m/Y H:i:s";
  const INT_FORMATS = [
    "year" => "Y",
    "month" => "m",
    "day" => "d",
    "hour" => "H",
    "minute" => "i",
    "second" => "s",
    "wday" => "N",
    "wnum" => "W",
  ];
  const STRING_FORMATS = [
    "timezone" => "P",
    "datetime" => "d/m/Y H:i:s",
    "date" => "d/m/Y",
  ];

  static function clone(DateTimeInterface $dateTime): self {
    if ($dateTime instanceof static) return clone $dateTime;
    $clone = new static();
    $clone->setTimestamp($dateTime->getTimestamp());
    $clone->setTimezone($dateTime->getTimezone());
    return $clone;
  }

  function __construct($datetime="now", DateTimeZone $timezone=null) {
    if ($datetime instanceof \DateTimeInterface) {
      if ($timezone === null) $timezone = $datetime->getTimezone();
      parent::__construct();
      $this->setTimestamp($datetime->getTimestamp());
      $this->setTimezone($timezone);
    } elseif (is_int($datetime)) {
      parent::__construct();
      $this->setTimestamp($datetime);
    } else {
      parent::__construct($datetime, $timezone);
    }
  }

  function diff($target, $absolute=false): DateInterval {
    return new DateInterval(parent::diff($target, $absolute));
  }

  function format($format=self::DEFAULT_FORMAT): string {
    return \DateTime::format($format);
  }

  function __toString(): string {
    return $this->format();
  }

  function __get($name) {
    if (array_key_exists($name, self::INT_FORMATS)) {
      return intval($this->format(self::INT_FORMATS[$name]));
    } elseif (array_key_exists($name, self::STRING_FORMATS)) {
      return $this->format(self::STRING_FORMATS[$name]);
    }
    throw new InvalidArgumentException("Unknown property $name");
  }
}
